"""
TimeSerie generators.

This module contains several generator
functions which generate TimeSerie objects
with different trends.
"""

import math

import numpy as np  # type: ignore
import pandas as pd  # type: ignore


from tsgen.time_serie import TimeSerie


def affine(start, end, freq, start_y, end_y) -> TimeSerie:
    """
    Generate a linear TimeSerie.
    """
    index = pd.date_range(start=start, end=end, freq=freq)
    return TimeSerie(
        index=index, y_values=np.linspace(start_y, end_y, len(index))
    )


def constant(start, end, freq, value) -> TimeSerie:
    """
    Generate a constant TimeSerie.
    """
    return affine(start, end, freq, value, value)


def cosine(start, end, freq, amp=1, n_periods=1) -> TimeSerie:
    """
    Generate a cosine TimeSerie.
    """
    index = pd.date_range(start=start, end=end, freq=freq)
    return TimeSerie(
        index=index,
        y_values=amp
        * np.cos(np.linspace(0, 2 * math.pi * n_periods, num=len(index))),
    )


def sine(start, end, freq, n_periods=1) -> TimeSerie:
    """
    Generate a sine TimeSerie.
    """
    index = pd.date_range(start=start, end=end, freq=freq)
    return TimeSerie(
        index=index,
        y_values=np.sin(
            np.linspace(0, 2 * math.pi * n_periods, num=len(index))
        ),
    )


def randn(start, end, freq, mean=0, std=1) -> TimeSerie:
    """
    Generate a random normally distributed TimeSerie.
    """
    index = pd.date_range(start=start, end=end, freq=freq)
    return TimeSerie(
        index=index, y_values=(std * np.random.randn(len(index)) + mean)
    )



"""TimeSerie Object."""

import pandas as pd  # type: ignore
import numpy as np  # type: ignore


class TimeSerie:
    """
    Represent a one-dimensional time serie.

    Parameters
    ----------
    index : pd.DatetimeIndex
        Time index.

    y_values : array_like
        One dimensional array_like object.

    Examples
    --------
    >>> start, end, freq = "2020", "2021", "1M"
    >>> index = pd.date_range(start=start, end=end, freq=freq)
    >>> ts = TimeSerie(
    ...     index=index,
    ...     y_values=np.linspace(1, 1, len(index))
    ... )

    >>> ts
                y_values
    2020-01-31  0.000000
    2020-02-29  0.090909
    2020-03-31  0.181818
    2020-04-30  0.272727
    2020-05-31  0.363636
    2020-06-30  0.454545
    2020-07-31  0.545455
    2020-08-31  0.636364
    2020-09-30  0.727273
    2020-10-31  0.818182
    2020-11-30  0.909091
    2020-12-31  1.000000

    >>> ts + 3 - ts
                y_values
    2020-01-31       3.0
    2020-02-29       3.0
    2020-03-31       3.0
    2020-04-30       3.0
    2020-05-31       3.0
    2020-06-30       3.0
    2020-07-31       3.0
    2020-08-31       3.0
    2020-09-30       3.0
    2020-10-31       3.0
    2020-11-30       3.0
    2020-12-31       3.0
    """

    def __init__(self, index, y_values):
        if not isinstance(index, pd.DatetimeIndex):
            raise TypeError("index should be a pandas.DatetimeIndex")
        self.index = index
        self.y_values = np.array(y_values)
        if len(y_values) != len(y_values):
            raise ValueError("index and y_values's shapes do not match")

    def to_frame(self):
        """
        Convert the TimeSerie to a pandas DataFrame
        """
        return pd.DataFrame({"y_values": self.y_values}, index=self.index)

    def plot(self):
        """
        Plot the TimeSerie using pandas
        `pd.Series.plot function
        <https://pandas.pydata.org/docs/reference/api/pandas.Series.plot.html>`_.
        """
        self.to_frame().y_values.plot()

    def __len__(self):
        return len(self.index)

    def __str__(self):
        return str(self.to_frame())

    def __repr__(self):
        return self.__str__()

    def __eq__(self, other):
        if not isinstance(other, TimeSerie):
            return False

        return (self.index == other.index).all() and (
            self.y_values == other.y_values
        ).all()

    def __mul__(self, other):
        if (
            (not isinstance(other, TimeSerie))
            and (not isinstance(other, int))
            and (not isinstance(other, float))
        ):
            raise TypeError("Wrong y_values")
        if (
            isinstance(other, TimeSerie)
            and not (self.index == other.index).all()
        ):
            raise ValueError("Indexes do not match")

        if isinstance(other, TimeSerie):
            return TimeSerie(
                index=self.index, y_values=(self.y_values * other.y_values)
            )

        return TimeSerie(index=self.index, y_values=(self.y_values * other))

    def __add__(self, other):
        if (
            (not isinstance(other, TimeSerie))
            and (not isinstance(other, int))
            and (not isinstance(other, float))
        ):
            raise TypeError("Wrong y_values")
        if (
            isinstance(other, TimeSerie)
            and not (self.index == other.index).all()
        ):
            raise ValueError("Indexes do not match")

        if isinstance(other, TimeSerie):
            return TimeSerie(
                index=self.index, y_values=(self.y_values + other.y_values)
            )

        return TimeSerie(index=self.index, y_values=(self.y_values + other))

    def __radd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        if isinstance(other, TimeSerie):
            negative_other = TimeSerie(
                index=self.index, y_values=(-1 * other.y_values)
            )
        else:
            negative_other = -1 * other

        return self + negative_other



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tsgen']

package_data = \
{'': ['*']}

install_requires = \
['coverage>=5.3,<6.0', 'numpy>=1.19.2,<2.0.0', 'pandas>=1.1.3,<2.0.0']

setup_kwargs = {
    'name': 'tsgen',
    'version': '0.3.0',
    'description': 'Time Series Generator',
    'long_description': '# tsgen\n\n[![CI/CD Status](https://github.com/MarwanDebbiche/tsgen/workflows/CI%2FCD/badge.svg?branch=master)](https://github.com/MarwanDebbiche/tsgen/actions?query=branch:master)\n[![Coverage Status](https://coveralls.io/repos/github/MarwanDebbiche/tsgen/badge.svg?branch=master)](https://coveralls.io/github/MarwanDebbiche/tsgen?branch=master)\n[![Latest Version](https://img.shields.io/pypi/v/tsgen.svg?color=blue)](https://pypi.python.org/pypi/tsgen)\n[![PyPI - Downloads](https://img.shields.io/pypi/dm/tsgen?label=pypi%20downloads)](https://pypi.org/project/tsgen/)\n![License](https://img.shields.io/github/license/MarwanDebbiche/tsgen)\n\ntsgen (for Time Series Generator) is a package developed for educational purposes that helps create unidimensional time series of different shapes.\n\n## Installation\n\nYou can install tsgen from PyPI using pip:\n\n```\npip install tsgen\n```\n\n## Getting started\n\n![Getting Started](./images/getting_started.png)\n\n<!-- See the [documentation]() for more details. -->\n',
    'author': 'Marwan Debbiche',
    'author_email': 'marwan.debbiche@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/MarwanDebbiche/tsgen',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
